;-----------------------------------
; A2D.ASM
;-----------------------------------
	#include <P12F509.inc>

	__CONFIG _WDT_OFF & _IntRC_OSC & _CP_OFF & _MCLRE_OFF
;	__CONFIG 0x0a
;
;-----------------------------------
; DEFINITION
;-----------------------------------
#define UARTPORT 	GPIO
#define TRISUARTPORT	TRIS
#define TXPIN  0	;GP0 OUT
#define RXPIN  1	;GP1 IN
#define CMPOUT 4	;GP4 OUT
#define THRCMP 3	;GP3 IN
#define DBG_O  5	;GP5 OUT
#define LED_O  2	;GP2 OUT	
#define TRISMASK  0x00
#define portmask  ((0x01 << RXPIN) | TRISMASK) & ~(0x01 << TXPIN)
;
MODE       EQU     1       ; If ( X_MODE==1) Then transmit LSB first
Nbit       EQU     1       ; if (X_Nbit==1) # of data bits ( Transmission ) 
                           ; is 8 else 7
STOPBITS   EQU  0     		; if Sbit2 = 0 then 1 Stop Bit else 2 Stop Bits
CLOCKSPEED EQU  4
BAUDRATE   EQU  9600
LSB        EQU     0
MSB        EQU     7
;BAUDRATE == 9600
BAUD_1     EQU     .34     ; 3+3X = CLKOUT/Baud (9600 baud, 4Mhz)
BAUD_4     EQU     .42     ; 3+3X = 1.25*CLKOUT/Baud
BAUD_X     EQU     .31     ; 11+3X = CLKOUT/Baud
BAUD_Y     EQU     .32     ; 9 +3X = CLKOUT/Baud
;
;-----------------------------------
;MACROS
;-----------------------------------
; Generic delay loop
Delay   MACRO
	LOCAL	LOOP
	MOVWF   DlyCnt
LOOP
	DECFSZ  DlyCnt,f
	GOTO    LOOP
	ENDM

; Specific delay loops... loads counter value then generic delay loop
Delay1  MACRO			; Start & stop bits 3+3X = CLKOUT/Baud
	MOVLW   BAUD_1
    	Delay
	ENDM
;
Delay4  MACRO			; 3+3X = 1.25*CLKOUT/Baud
	MOVLW   BAUD_4           
    	Delay
	ENDM
;
DelayX  MACRO			; 11+3X = CLKOUT/Baud
    	MOVLW   BAUD_X
    	Delay
    	ENDM
;
DelayY  MACRO			; 9 +3X = CLKOUT/Baud
    	MOVLW   BAUD_Y
    	Delay
    	ENDM
;
	ORG	07H
;-----------------------------------
; WORK AREA
;-----------------------------------
PrtCounter    RES   1		;07
Digit   	     RES   1		;08
counter 	     RES   1		;09
PrintValue    RES   2		;0a
Result_l      RES   1		;0c
Result_h      RES   1		;0d
D_COUNTER     RES   2		;0e
RcvReg  	     RES   1		;10 Data received
XmtReg  	     RES   1		;11
Count   	     RES   1		;12 Counter for #of Bits Transmitted
DlyCnt  	     RES   1		;13
ACCaLO  	     RES   1		;14
ACCaHI  	     RES   1		;15
ACCbLO  	     RES   1		;16
ACCbHI  	     RES   1		;17
Flags         RES   1		;18 software flags
;
WORK          RES   1		;19
WORK_CNT      RES   1      ;1a
LED_FLG       RES   1		;1b
CMCON_DUM     RES   1      ;1c dummy
;
	ORG	0
;-----------------------------------
; STARTUP
;-----------------------------------
STARTUP
     MOVWF   OSCCAL		;
     BCF     STATUS,PA0	;program BANK 0
     GOTO    START		;
;-----------------------------------
; Main loop
;-----------------------------------
START
     CALL    PORTINIT		;
     CALL    OPENUART		;
RESTART
     MOVLW   .1			;
     MOVWF   PrtCounter	;
LOOP
     BSF     GPIO,DBG_O	;
     CALL    DELTASIGA2D	;
     BCF     GPIO,DBG_O	;
     MOVF    Result_h,w	;
     MOVWF   PrintValue+1	;
     MOVF    Result_l,w	;
     MOVWF   PrintValue	;
     CALL    PRINT16b		;
     DECFSZ  PrtCounter,f	;
     GOTO    LOOP			;
     MOVLW   0x0d			;CR
     CALL    PUTCUART		;
     MOVLW   0x0a			;LF
     CALL    PUTCUART		;
     DECFSZ  WORK_CNT		;Ƃɂ޸
     GOTO    RESTART		;
LED_CHG
     MOVFW   LED_FLG		;
     ANDLW   0xff			;
     BTFSC   STATUS,Z		;
     GOTO    LED_ON		;
     MOVLW   0			;
     MOVWF   LED_FLG		;
     BSF     GPIO,LED_O	;
     GOTO    RESTART		;
LED_ON
     MOVLW   1			;
     MOVWF   LED_FLG		;
     BCF     GPIO,LED_O	;
     GOTO    RESTART		;
;-----------------------------------
;PORT INIT
;-----------------------------------
PORTINIT
     MOVLW   B'11000000'	;
     OPTION_REG			;
     NOP
     MOVLW   B'00001010'	;
     TRIS    GPIO			;
     NOP
     BSF     GPIO,LED_O	;LED OFF
     NOP					;
     BCF     GPIO,DBG_O	;DEBUG OUT LOW
     NOP					;
     BCF     GPIO,LED_O	;
     RETURN				;
;-----------------------------------
;SERIAL PORT OPEN
;-----------------------------------
OPENUART
     BSF     UARTPORT,TXPIN	; Start it off high
     RETLW   0				;
;-----------------------------------
;AD CONVERT
;-----------------------------------
DELTASIGA2D
     CLRF    D_COUNTER		;
     CLRF    D_COUNTER+1		;
     CLRF    Result_l			;
     CLRF    Result_h			;
D_LOOP
     BTFSC   GPIO,THRCMP		; ( THRESH HOLD )COMPARATOR INPUT?
     GOTO    COMPLOW         	; Go the low route
COMPHIGH
     NOP                     	; necessary to keep timing even
     BCF     GPIO,CMPOUT      	; PortA.3 = 0
     INCFSZ  Result_l,f      	; bump counter
     GOTO    EAT2CYCLES      	; 
     INCF    Result_h,f      	; 
     GOTO    ENDLOOP         	; 
COMPLOW
     BSF     GPIO,CMPOUT       	; Comparator is low
     NOP                     	; necessary to keep timing even
     GOTO    EAT2CYCLES      	; same here 
EAT2CYCLES
     GOTO    ENDLOOP         	; eat 2 more cycles
ENDLOOP
     INCFSZ  D_COUNTER,f       ; Count this lap through the loop.
     GOTO    EAT5CYCLES      	; 
     INCF    D_COUNTER+1,f     ; 
     MOVF    D_COUNTER+1,w     ; 
     ANDLW   0x04            	; Are we done? (We're done when bit2 of
     BTFSC   STATUS,Z        	; the high order byte overflows to 1).
     GOTO    D_LOOP			;                       
     GOTO    EXIT				;
EAT5CYCLES
     GOTO    $+1             	; more wasted time to keep the loops even
     NOP                     	; 
     GOTO    D_LOOP			; 
EXIT
     RETURN
;-----------------------------------
;16bit Value Print
;-----------------------------------
PRINT16b
     CLRF	Digit			; used to select power of 10
NEXTDIGIT
     CLRF	counter			;
LOOP1000
     MOVF	PrintValue,w		;
     MOVWF	ACCbLO			;
     MOVF	PrintValue+1,w	;
     MOVWF	ACCbHI			;
;
     MOVF	Digit,w			;
     CALL	getpower			; low order bits of 1000
     MOVWF	ACCaLO			;
     INCF	Digit,w			;
     CALL	getpower			;
     MOVWF	ACCaHI			;
;
     CALL	SUB16			;
;
     BTFSS	STATUS,C			; was carry set?
     GOTO	PRINT1000		;
     INCF	counter,f		;
     MOVF	ACCbLO,w			;
     MOVWF	PrintValue		;
     MOVF	ACCbHI,w			;
     MOVWF	PrintValue+1		;
     GOTO	LOOP1000			;
PRINT1000
;     MOVF	counter,w		;
;ADDLWȂ̂
;    ADDLW	'0'				; make it a number
	MOVLW	'0'				; w <- '0'
	ADDWF	counter,w		; w <- w + '0'
;
     CALL	PUTCUART			; w -> send
     MOVLW	2				;
     ADDWF	Digit,f			;
	
     MOVLW	6				;
     SUBWF	Digit,w			; are we done?
     BTFSS	STATUS,Z			;
     GOTO	NEXTDIGIT		;
PRINT1						; Whatever's left are the 1's digit
;     MOVF	PrintValue,w		;
;ADDLWȂ̂
;    ADDLW	'0'				; make it a number
	MOVLW	'0'				; w <- '0'
	ADDWF	PrintValue,w		; w <- w + '0'
;
     CALL	PUTCUART			; w -> send
     RETURN					;
getpower
     ADDWF	PCL,f			;
     RETLW	0xe8				;
     RETLW	0x03   			; 1000
     RETLW	.100				;
     RETLW	.0     			; 100
     RETLW	.10				;
     RETLW	.0     			; 10
;-----------------------------------
;SERIAL SEND
;-----------------------------------
PUTCUART
     MOVWF	XmtReg			;
     MOVLW   8				;
     MOVWF   Count			;
;
     BCF     UARTPORT, TXPIN   ; Send Start Bit
     Delay1          			; (start bit) 52 uS (19200 baud)
X_next  
     BCF     STATUS,C			;
     RRF     XmtReg,f     		; Conditional Assembly
;
     BTFSC   STATUS,C			;
     BSF     UARTPORT, TXPIN	;
     BTFSS   STATUS,C			;
     BCF     UARTPORT, TXPIN	;
     DelayX					;
     DECFSZ  Count,f			;
     GOTO    X_next			;
     BSF     UARTPORT, TXPIN   ; Send Stop Bit
     Delay1					;
     RETLW	0x01				; return to calling program
;-----------------------------------
;SERIAL RECEIVE
;-----------------------------------
GETCUART
     CLRF    RcvReg			; Clear RcvReg
     BCF     STATUS,C			; Clear the carry bit (assume nothing rx'd)
     BTFSC   UARTPORT,RXPIN	; check for a Start Bit
     RETLW	0				; no char received.. return null.
     Delay4                  	; delay for 1
     MOVLW   8               	; 8 Data bits
     MOVWF   Count			;
R_next  
     BCF     STATUS,C			;
     RRF     RcvReg,f     		; to set if MSB first or LSB first
     BTFSC   UARTPORT, RXPIN	;
     BSF     RcvReg,LSB		;
     DelayY					;
     DECFSZ  Count,f			;
     GOTO    R_next			;
     BSF     STATUS,C			;
     RETLW   0x01				;
;-----------------------------------
;16bit SUB
;-----------------------------------
SUB16	
     COMF    ACCaLO, F       	; At first negate ACCa; Then add
     INCF    ACCaLO, F		; negate ACCa ( -ACCa -> ACCa )
     BTFSC   STATUS,Z			;
     DECF    ACCaHI, F		;
     COMF    ACCaHI, F		;
ADD16   
     BCF     Flags,C			;
     MOVF    ACCaLO,w        	; Addition ( ACCb + ACCa -> ACCb )
     ADDWF   ACCbLO, F       	; add lsb
     BTFSS   STATUS,C    		; did that overflow?
     GOTO	_no_overflow		; no: skip on down to do hi byte
     BCF     STATUS,C			;
     INCF    ACCbHI, F		; yes: add carry into ACCbHi
     BTFSC   STATUS,C		; did that overflow?
     BSF     Flags,C			; set the software overflow flag
_no_overflow					; No: continue on with hi byte
     MOVF     ACCaHI,w		;
     ADDWF    ACCbHI, F       	; add msb
     BTFSC    Flags,C			; Was there a prior overflow?
     BSF	     STATUS,C			; Yes: set overflow (carry) bit.
     RETLW    0				;

	END
